@@ -1,5 +1,5 @@
 require(data.table)
 options(warn=2)
-test.data.table()
+test.data.table(verbose=TRUE)   # verbose to check the verbose messages run ok
 test.data.table()  # calling it again can reveal some memory bugs
 
